library FirstLib;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Dialogs,
  Windows,
  ActiveX,
  Classes,
  Forms,
  Controls,
  Unit1 in 'Unit1.pas' {Form1},
  Unit2 in 'Unit2.pas' {Form2};

{$R *.res}
type
  TReturnNextMethod=function:integer; stdcall;
  TReturnNextMethodObject=function:integer of object;
   stdcall;
  TReturnNextMethodExport=function:integer; stdcall;

var
  {    !}
  Buffer:array [0..1000] of Char;
  C:array[0..1000] of char;

function AddOne(N: Integer): Integer; stdcall; export;
begin
  Result := N + 1;
end;

procedure SendString(P: PChar); stdcall; export;
var
  S: String;
begin
  S := StrPas(P);
  ShowMessage(S);
end;

procedure ReceiveString(P: PChar; Size: Integer); stdcall;  export;
var
  S: String;
  N: Integer;
begin
  FillMemory(P, Size, 0);
  if InputQuery(' ,     ',
     '', S) then
     begin
       N := Length(S);
       if N > Size then N := Size;
       if N > 0 then System.Move(S[1], P[0], N);
     end;
end;

procedure ReceiveBuffer(var P: PChar); stdcall;  export;
var
  S: String;
  N: Integer;
begin
  FillMemory(@Buffer[0], SizeOf(Buffer), 0);
  if InputQuery(' ,     ',
     '', S) then
     begin
       N := Length(S);
       if N > SizeOf(Buffer) - 1 then N := SizeOf(Buffer) - 1;
       if N > 0 then System.Move(S[1], Buffer[0], N);
     end;
  P := @Buffer[0];
end;

procedure ReceiveWinAPI(var HMem:integer); stdcall; export;
var
  S:string;
  N:Integer;
  P:PChar;
begin
  HMem := 0;
  if InputQuery(' ,     ',
     '', S) then
  begin
    N:=length(S);
    if N>0 then begin
      HMem := GlobalAlloc(GMEM_DDESHARE or GMEM_MOVEABLE
        or GMEM_ZEROINIT,N+1);
      if HMem>0 then begin
        P:=GlobalLock(HMem);
        if Assigned(P) then System.Move(S[1],P[0],N);
      end;
    end;
  end;
end;

procedure ReceiveCOMAPI(var P:pointer); stdcall; export;
var
  S:string;
  N:integer;
begin
  P := nil;
  if InputQuery(' ,     ',
     '', S) then
  begin
    N := length(S);
    if N>0 then begin
      P:=CoTaskMemAlloc(length(S)+1);
      if Assigned(P) then begin
        FillMemory(P,N+1,0);
        System.Move(S[1],P^,N);
      end;
    end;
  end;
end;

function CalculateSum(ReturnCallback:pointer):integer; stdcall;
  export;
var
  N:integer;
  ReturnNext:TReturnNextMethod;
begin
  Result := 0;
  if ReturnCallback=nil then Exit;
  N := 0;
  ReturnNext:=TReturnNextMethod(ReturnCallback);
  while N>=0 do begin
    N := ReturnNext;
    if N>=0 then Result:=Result+N;
  end;
end;

function CalculateSumObject(Method:TMethod):integer;
   stdcall; export;
var
  N:integer;
  ReturnNext:TReturnNextMethodObject;
begin
  Result := 0;
  N := 0;
  ReturnNext := TReturnNextMethodObject(Method);
  while N>=0 do begin
    N := ReturnNext;
    if N>=0 then Result := Result+N;
  end;
end;

function CalculateSumExport(MethodName:pchar):integer;
  stdcall; export;
var
  ReturnNextExport:TReturnNextMethodExport;
  N:integer;
begin
  Result := 0;
  N := 0;
  //   - , GetModuleHandle 
  //  ,   
  //   .
  ReturnNextExport:=GetProcAddress(GetModuleHandle(NIL),
    MethodName);
  while N>=0 do begin
    N := ReturnNextExport;
    if N>=0 then Result:=Result+N;
  end;
end;

function ExecDialog(AppHandle:THandle;
  var PictName:PChar):boolean; stdcall; export;
var
  FDialog:TForm1;
begin
  FDialog := nil;
  PictName := nil;
  Result := False;
  Application.Handle := AppHandle;
  try
    FDialog := TForm1.Create(Application);
    if FDialog.ShowModal = mrOK then begin
      FillMemory(@C[0], SizeOf(C), 0);
      if length(FDialog.OpenPictureDialog1.FileName) > 0 then
          StrPCopy(C,FDialog.OpenPictureDialog1.FileName);
      PictName := @C[0];
      Result := True;
    end;
    FDialog.Free;
    FDialog := nil;
  except
    On E:Exception do begin
      ShowMessage(E.Message);
      if Assigned(FDialog) then FDialog.Free;
    end;
  end;
  Application.ProcessMessages;
  Application.Handle := 0;
end;

procedure ShowNonModalForm(AppHandle: THandle); stdcall;
begin
  Application.Handle:=AppHandle;
  with TForm2.Create(Application) do Show;
end;

exports
  AddOne index 1 name 'CalculateSum',
  SendString index 2 name 'SendString',
  ReceiveString index 3 name 'ReceiveString',
  ReceiveBuffer index 4 name 'ReceiveBuffer',
  ReceiveWinAPI index 5 name 'ReceiveWinAPI',
  ReceiveCOMAPI index 6 name 'ReceiveCOMAPI',
  CalculateSum index 7 name 'Sum',
  CalculateSumObject index 8 name 'SumObject',
  CalculateSumExport index 9 name 'SumExport',
  ExecDialog index 10 name 'ExecDialog',
  ShowNonModalForm index 11 name 'ShowNonModalForm',
  DynNonmodal index 12 name 'DynNonmodal';
begin
end.
